using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Security;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect.BaseClasses;

public abstract class FunctionalBaseClass {
    protected readonly FitConnectApiService ApiService;
    public readonly IEncryption Encryption;
    protected readonly ILogger? Logger;
    internal Client Owner { get; set; }


    /// <summary>
    ///     Constructor for the FunctionalBaseClass
    /// </summary>
    /// <param name="logger">ILogger implementation</param>
    /// <param name="endpoints">FitConnect endpoints</param>
    /// <param name="certificate">The Encryption certificate</param>
    /// <example>
    ///     new Sender(logger, FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development))
    /// </example>
    protected FunctionalBaseClass(ILogger? logger, FitConnectEndpoints? endpoints,
        X509Certificate2? certificate) {
        Endpoints = endpoints ??
                    FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development);

        Logger = logger;
        Encryption = new RsaEncryption(logger, certificate);
        ApiService = new FitConnectApiService(Endpoints, logger);
    }

    public FitConnectEndpoints Endpoints { get; }


    /// <summary>
    /// Requesting the OAuth-Token
    /// </summary>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="scope"></param>
    /// <returns></returns>
    public Task<OAuthAccessToken?> AuthenticateAsync(string clientId, string clientSecret, string? scope) {
        return ApiService.OAuthService.AuthenticateAsync(clientId, clientSecret, scope);
    }

    /// <summary>
    ///     Receive the SET data
    /// </summary>
    /// <returns></returns>
    public Task<SecurityEventToken> GetSetDataAsync() {
        throw new NotImplementedException();
    }
}
