using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect;

public class FluentSender : Sender {
    private OAuthAccessToken? token;
    public PublicKey PublicKey { get; set; }
    public CreateSubmissionDto? NewSubmission { get; set; }

    public static Sender Create() {
        throw new NotImplementedException();
    }

    public FluentSender Authenticate(string clientId, string clientSecret, string? scope = null) {
        Owner.ClientId = clientId;
        Owner.ClientSecret = clientSecret;

        token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;
        return this;
    }

    /// <summary>
    /// Creates a new <see cref="Submission"/> on the FitConnect server.
    /// </summary>
    /// <param name="submission"></param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    /// <exception cref="ArgumentException"></exception>
    public FluentSender CreateSubmission(Submission submission) {
        if (token == null)
            throw new InvalidOperationException("You must authenticate first.");

        if (!submission.IsSubmissionReadyToAdd(out var errorMessage))
            throw new ArgumentException($"Submission is not ready to add. {errorMessage}");

        NewSubmission = AddSubmission(submission);

        return this;
    }

    public FluentSender SendSubmission() {
        throw new NotImplementedException();
    }

    public FluentSender UploadAttachments(List<Attachment> attachments) {
        if (NewSubmission == null)
            throw new InvalidOperationException("You must create a submission first.");

        if (attachments.Count == 0)
            return this;

        var encryptedAttachments = Encrypt(PublicKey, attachments);
        UploadAttachmentsAsync(encryptedAttachments).Wait();

        return this;
    }

    public FluentSender SendSubmission(Metadata metadata, Data? data = null) {
        throw new NotImplementedException();
    }

    public FluentSender(FitConnectEndpoints endpoints, X509Certificate2? certificate = null,
        ILogger? logger = null) : base(endpoints, certificate, logger) {
        PublicKey = certificate.PublicKey;
    }
}
