using FitConnect.Services.Models;

namespace FitConnect.Models;

public record Submission {
    public string? Id { get; set; }
    public string DestinationId { get; init; }
    public string? CaseId { get; set; }

    public List<Attachment> Attachments { get; set; } = new();

    public ServiceType ServiceType { get; init; }

    public Callback? Callback { get; set; }

    public bool IsSubmissionReadyToAdd(out string? error) {
        error = null;
        return true;
    }

    public bool IsSubmissionReadyToSend() {
        return true;
    }


    /// <summary>
    ///     Fachdaten
    /// </summary>
    public string? Data { get; set; }

    public static explicit operator Submission(SubmissionForPickupDto dto) {
        return new() {
            Id = dto.SubmissionId,
            Callback = null,
            DestinationId = dto.DestinationId,
            ServiceType = null
        };
    }

    public static explicit operator Submission(SubmissionDto dto) {
        return new() {
            Id = dto.SubmissionId,
            Callback = (Callback)dto.Callback,
            DestinationId = dto.DestinationId,
            ServiceType = (ServiceType)dto.ServiceType
        };
    }

    public static explicit operator CreateSubmissionDto(Submission sub) {
        return new() {
            AnnouncedAttachments = sub.Attachments.Select(a => a.Id).ToList(),
            DestinationId = sub.DestinationId,
            ServiceType = (ServiceTypeDto)sub.ServiceType,
            Callback = (CallbackDto)sub.Callback
        };
    }
}
